from webob import Response as BaseResponse


class Response(BaseResponse):
    """Application response implementation.

    There is no custom logic here at the moment, but there will be in the future.
    """
