from webob import Request as BaseRequest


class Request(BaseRequest):
    """Application request implementation.

    There is no custom logic here at the moment, but there will be in the future.
    """
